@@ -1,5 +1,3 @@
-from __future__ import print_function, absolute_import
-
 import ctypes
 from ctypes import CFUNCTYPE, c_int
 from ctypes.util import find_library
@@ -14,10 +12,20 @@ import unittest
 from contextlib import contextmanager
 from tempfile import mkstemp
 
-from llvmlite import six, ir
+from llvmlite import ir
 from llvmlite import binding as llvm
 from llvmlite.binding import ffi
-from . import TestCase
+from llvmlite.tests import TestCase
+
+
+def clean_string_whitespace(x: str) -> str:
+    # Remove trailing whitespace from the end of each line
+    x = re.sub(r"\s+$", "", x, flags=re.MULTILINE)
+    # Remove intermediate blank lines
+    x = re.sub(r"\n\s*\n", r"\n", x, flags=re.MULTILINE)
+    # Remove extraneous whitespace from the beginning and end of the string
+    x = x.strip()
+    return x
 
 
 # arvm7l needs extra ABI symbols to link successfully
@@ -39,6 +47,7 @@ def no_de_locale():
 
 asm_sum = r"""
     ; ModuleID = '<string>'
+    source_filename = "asm_sum.c"
     target triple = "{triple}"
     %struct.glob_type = type {{ i64, [2 x i64]}}
 
@@ -126,6 +135,136 @@ asm_inlineasm = r"""
     }}
     """
 
+asm_inlineasm2 = """
+    ; ModuleID = '<string>'
+    target triple = "{triple}"
+
+    define void @inlineme() {{
+        ret void
+    }}
+
+    define i32 @caller(i32 %.1, i32 %.2) {{
+    entry:
+      %stack = alloca i32
+      store i32 %.1, i32* %stack
+      br label %main
+    main:
+      %loaded = load i32, i32* %stack
+      %.3 = add i32 %loaded, %.2
+      %.4 = add i32 0, %.3
+      call void @inlineme()
+      ret i32 %.4
+    }}
+"""
+
+asm_inlineasm3 = """
+; ModuleID = 'test.c'
+source_filename = "test.c"
+target triple = "{triple}"
+
+; Function Attrs: noinline nounwind optnone ssp uwtable
+define void @inlineme() noinline !dbg !15 {{
+  ret void, !dbg !18
+}}
+
+; Function Attrs: noinline nounwind optnone ssp uwtable
+define i32 @foo(i32 %0, i32 %1) !dbg !19 {{
+  %3 = alloca i32, align 4
+  %4 = alloca i32, align 4
+  store i32 %0, i32* %3, align 4
+  call void @llvm.dbg.declare(metadata i32* %3, metadata !23, metadata !DIExpression()), !dbg !24
+  store i32 %1, i32* %4, align 4
+  call void @llvm.dbg.declare(metadata i32* %4, metadata !25, metadata !DIExpression()), !dbg !26
+  call void @inlineme(), !dbg !27
+  %5 = load i32, i32* %3, align 4, !dbg !28
+  %6 = load i32, i32* %4, align 4, !dbg !29
+  %7 = add nsw i32 %5, %6, !dbg !30
+  ret i32 %7, !dbg !31
+}}
+
+; Function Attrs: nofree nosync nounwind readnone speculatable willreturn
+declare void @llvm.dbg.declare(metadata, metadata, metadata) #1
+
+attributes #1 = {{ nofree nosync nounwind readnone speculatable willreturn }}
+
+!llvm.module.flags = !{{!1, !2, !3, !4, !5, !6, !7, !8, !9, !10}}
+!llvm.dbg.cu = !{{!11}}
+!llvm.ident = !{{!14}}
+
+!0 = !{{i32 2, !"SDK Version", [2 x i32] [i32 12, i32 3]}}
+!1 = !{{i32 7, !"Dwarf Version", i32 4}}
+!2 = !{{i32 2, !"Debug Info Version", i32 3}}
+!3 = !{{i32 1, !"wchar_size", i32 4}}
+!4 = !{{i32 1, !"branch-target-enforcement", i32 0}}
+!5 = !{{i32 1, !"sign-return-address", i32 0}}
+!6 = !{{i32 1, !"sign-return-address-all", i32 0}}
+!7 = !{{i32 1, !"sign-return-address-with-bkey", i32 0}}
+!8 = !{{i32 7, !"PIC Level", i32 2}}
+!9 = !{{i32 7, !"uwtable", i32 1}}
+!10 = !{{i32 7, !"frame-pointer", i32 1}}
+!11 = distinct !DICompileUnit(language: DW_LANG_C99, file: !12, producer: "Apple clang version 13.1.6 (clang-1316.0.21.2.3)", isOptimized: false, runtimeVersion: 0, emissionKind: FullDebug, enums: !13, splitDebugInlining: false, nameTableKind: None, sysroot: "/Applications/Xcode.app/Contents/Developer/Platforms/MacOSX.platform/Developer/SDKs/MacOSX.sdk", sdk: "MacOSX.sdk")
+!12 = !DIFile(filename: "test.c", directory: "/")
+!13 = !{{}}
+!14 = !{{!"Apple clang version 13.1.6 (clang-1316.0.21.2.3)"}}
+!15 = distinct !DISubprogram(name: "inlineme", scope: !12, file: !12, line: 1, type: !16, scopeLine: 1, flags: DIFlagPrototyped, spFlags: DISPFlagDefinition, unit: !11, retainedNodes: !13)
+!16 = !DISubroutineType(types: !17)
+!17 = !{{null}}
+!18 = !DILocation(line: 1, column: 22, scope: !15)
+!19 = distinct !DISubprogram(name: "foo", scope: !12, file: !12, line: 3, type: !20, scopeLine: 3, flags: DIFlagPrototyped, spFlags: DISPFlagDefinition, unit: !11, retainedNodes: !13)
+!20 = !DISubroutineType(types: !21)
+!21 = !{{!22, !22, !22}}
+!22 = !DIBasicType(name: "int", size: 32, encoding: DW_ATE_signed)
+!23 = !DILocalVariable(name: "a", arg: 1, scope: !19, file: !12, line: 3, type: !22)
+!24 = !DILocation(line: 3, column: 13, scope: !19)
+!25 = !DILocalVariable(name: "b", arg: 2, scope: !19, file: !12, line: 3, type: !22)
+!26 = !DILocation(line: 3, column: 20, scope: !19)
+!27 = !DILocation(line: 4, column: 5, scope: !19)
+!28 = !DILocation(line: 5, column: 12, scope: !19)
+!29 = !DILocation(line: 5, column: 16, scope: !19)
+!30 = !DILocation(line: 5, column: 14, scope: !19)
+!31 = !DILocation(line: 5, column: 5, scope: !19)
+"""  # noqa E501
+
+licm_asm = r"""
+; ModuleID = "<string>"
+target triple = "{triple}"
+    
+define double @licm(i32 %0) {{
+  %2 = alloca i32, align 4
+  %3 = alloca double, align 8
+  %4 = alloca i32, align 4
+  %5 = alloca double, align 8
+  store i32 %0, i32* %2, align 4
+  store double 0.000000e+00, double* %3, align 8
+  store i32 0, i32* %4, align 4
+  br label %6
+
+6:                                                ; preds = %14, %1
+  %7 = load i32, i32* %4, align 4
+  %8 = load i32, i32* %2, align 4
+  %9 = icmp slt i32 %7, %8
+  br i1 %9, label %10, label %17
+
+10:                                               ; preds = %6
+  store double 7.000000e+00, double* %5, align 8
+  %11 = load double, double* %5, align 8
+  %12 = load double, double* %3, align 8
+  %13 = fadd double %12, %11
+  store double %13, double* %3, align 8
+  br label %14
+
+14:                                               ; preds = %10
+  %15 = load i32, i32* %4, align 4
+  %16 = add nsw i32 %15, 1
+  store i32 %16, i32* %4, align 4
+  br label %6
+
+17:                                               ; preds = %6
+  %18 = load double, double* %3, align 8
+  ret double %18
+}}
+"""  # noqa E501
+
 asm_global_ctors = r"""
     ; ModuleID = "<string>"
     target triple = "{triple}"
@@ -153,7 +292,58 @@ asm_global_ctors = r"""
 
     @llvm.global_ctors = appending global [1 x {{i32, void ()*, i8*}}] [{{i32, void ()*, i8*}} {{i32 0, void ()* @ctor_A, i8* null}}]
     @llvm.global_dtors = appending global [1 x {{i32, void ()*, i8*}}] [{{i32, void ()*, i8*}} {{i32 0, void ()* @dtor_A, i8* null}}]
-    """
+    """  # noqa E501
+
+
+asm_nonalphanum_blocklabel = """; ModuleID = ""
+target triple = "unknown-unknown-unknown"
+target datalayout = ""
+
+define i32 @"foo"()
+{
+"<>!*''#":
+  ret i32 12345
+}
+"""  # noqa W291 # trailing space needed for match later
+
+
+riscv_asm_ilp32 = [
+    'addi\tsp, sp, -16',
+    'sw\ta1, 8(sp)',
+    'sw\ta2, 12(sp)',
+    'fld\tft0, 8(sp)',
+    'fmv.w.x\tft1, a0',
+    'fcvt.d.s\tft1, ft1',
+    'fadd.d\tft0, ft1, ft0',
+    'fsd\tft0, 8(sp)',
+    'lw\ta0, 8(sp)',
+    'lw\ta1, 12(sp)',
+    'addi\tsp, sp, 16',
+    'ret'
+]
+
+
+riscv_asm_ilp32f = [
+    'addi\tsp, sp, -16',
+    'sw\ta0, 8(sp)',
+    'sw\ta1, 12(sp)',
+    'fld\tft0, 8(sp)',
+    'fcvt.d.s\tft1, fa0',
+    'fadd.d\tft0, ft1, ft0',
+    'fsd\tft0, 8(sp)',
+    'lw\ta0, 8(sp)',
+    'lw\ta1, 12(sp)',
+    'addi\tsp, sp, 16',
+    'ret'
+]
+
+
+riscv_asm_ilp32d = [
+    'fcvt.d.s\tft0, fa0',
+    'fadd.d\tfa0, ft0, fa1',
+    'ret'
+]
+
 
 asm_attributes = r"""
 declare void @a_readonly_func(i8 *) readonly
@@ -161,6 +351,49 @@ declare void @a_readonly_func(i8 *) readonly
 declare i8* @a_arg0_return_func(i8* returned, i32*)
 """
 
+
+# This produces the following output from objdump:
+#
+# $ objdump -D 632.elf
+#
+# 632.elf:     file format elf64-x86-64
+#
+#
+# Disassembly of section .text:
+#
+# 0000000000000000 <__arybo>:
+#    0:	48 c1 e2 20          	shl    $0x20,%rdx
+#    4:	48 09 c2             	or     %rax,%rdx
+#    7:	48 89 d0             	mov    %rdx,%rax
+#    a:	48 c1 c0 3d          	rol    $0x3d,%rax
+#    e:	48 31 d0             	xor    %rdx,%rax
+#   11:	48 b9 01 20 00 04 80 	movabs $0x7010008004002001,%rcx
+#   18:	00 10 70
+#   1b:	48 0f af c8          	imul   %rax,%rcx
+
+issue_632_elf = \
+    "7f454c4602010100000000000000000001003e00010000000000000000000000000000" \
+    "0000000000e0000000000000000000000040000000000040000500010048c1e2204809" \
+    "c24889d048c1c03d4831d048b90120000480001070480fafc800000000000000000000" \
+    "0000000000000000000000000000002f0000000400f1ff000000000000000000000000" \
+    "00000000070000001200020000000000000000001f00000000000000002e7465787400" \
+    "5f5f617279626f002e6e6f74652e474e552d737461636b002e737472746162002e7379" \
+    "6d746162003c737472696e673e00000000000000000000000000000000000000000000" \
+    "0000000000000000000000000000000000000000000000000000000000000000000000" \
+    "00000000000000001f0000000300000000000000000000000000000000000000a80000" \
+    "0000000000380000000000000000000000000000000100000000000000000000000000" \
+    "000001000000010000000600000000000000000000000000000040000000000000001f" \
+    "000000000000000000000000000000100000000000000000000000000000000f000000" \
+    "01000000000000000000000000000000000000005f0000000000000000000000000000" \
+    "0000000000000000000100000000000000000000000000000027000000020000000000" \
+    "0000000000000000000000000000600000000000000048000000000000000100000002" \
+    "00000008000000000000001800000000000000"
+
+
+issue_632_text = \
+    "48c1e2204809c24889d048c1c03d4831d048b90120000480001070480fafc8"
+
+
 class BaseTest(TestCase):
 
     def setUp(self):
@@ -190,9 +423,9 @@ class BaseTest(TestCase):
             mod = self.module()
         return mod.get_global_variable(name)
 
-    def target_machine(self):
+    def target_machine(self, *, jit):
         target = llvm.Target.from_default_triple()
-        return target.create_target_machine()
+        return target.create_target_machine(jit=jit)
 
 
 class TestDependencies(BaseTest):
@@ -200,8 +433,10 @@ class TestDependencies(BaseTest):
     Test DLL dependencies are within a certain expected set.
     """
 
-    @unittest.skipUnless(sys.platform.startswith('linux'), "Linux-specific test")
-    @unittest.skipUnless(os.environ.get('LLVMLITE_DIST_TEST'), "Distribution-specific test")
+    @unittest.skipUnless(sys.platform.startswith('linux'),
+                         "Linux-specific test")
+    @unittest.skipUnless(os.environ.get('LLVMLITE_DIST_TEST'),
+                         "Distribution-specific test")
     def test_linux(self):
         lib_path = ffi.lib._name
         env = os.environ.copy()
@@ -235,6 +470,79 @@ class TestDependencies(BaseTest):
                 self.fail("unexpected dependency %r in %r" % (dep, deps))
 
 
+class TestRISCVABI(BaseTest):
+    """
+    Test calling convention of floating point arguments of RISC-V
+    using different ABI.
+    """
+    triple = "riscv32-unknown-linux"
+
+    def setUp(self):
+        super().setUp()
+        llvm.initialize_all_targets()
+        llvm.initialize_all_asmprinters()
+
+    def check_riscv_target(self):
+        try:
+            llvm.Target.from_triple(self.triple)
+        except RuntimeError as e:
+            if "No available targets are compatible with triple" in str(e):
+                self.skipTest("RISCV target unsupported by linked LLVM.")
+            else:
+                raise e
+
+    def riscv_target_machine(self, **kwarg):
+        lltarget = llvm.Target.from_triple(self.triple)
+        return lltarget.create_target_machine(**kwarg)
+
+    def fpadd_ll_module(self):
+        f64 = ir.DoubleType()
+        f32 = ir.FloatType()
+        fnty = ir.FunctionType(f64, (f32, f64))
+        module = ir.Module()
+        func = ir.Function(module, fnty, name="fpadd")
+        block = func.append_basic_block()
+        builder = ir.IRBuilder(block)
+        a, b = func.args
+        arg0 = builder.fpext(a, f64)
+        result = builder.fadd(arg0, b)
+        builder.ret(result)
+
+        llmod = llvm.parse_assembly(str(module))
+        llmod.verify()
+        return llmod
+
+    def break_up_asm(self, asm):
+        asm_list = []
+        for line in asm.splitlines():
+            s_line = line.strip()
+            if not (s_line.startswith(".") or s_line.startswith("fpadd")
+                    or s_line == ""):
+                asm_list.append(s_line)
+        return asm_list
+
+    def test_rv32d_ilp32(self):
+        self.check_riscv_target()
+        llmod = self.fpadd_ll_module()
+        target = self.riscv_target_machine(features="+f,+d")
+        self.assertEqual(self.break_up_asm(target.emit_assembly(llmod)),
+                         riscv_asm_ilp32)
+
+    def test_rv32d_ilp32f(self):
+        self.check_riscv_target()
+        llmod = self.fpadd_ll_module()
+        target = self.riscv_target_machine(features="+f,+d", abiname="ilp32f")
+        self.assertEqual(self.break_up_asm(target.emit_assembly(llmod)),
+                         riscv_asm_ilp32f)
+
+    def test_rv32d_ilp32d(self):
+        self.check_riscv_target()
+        llmod = self.fpadd_ll_module()
+        target = self.riscv_target_machine(features="+f,+d", abiname="ilp32d")
+        self.assertEqual(self.break_up_asm(target.emit_assembly(llmod)),
+                         riscv_asm_ilp32d)
+
+
 class TestMisc(BaseTest):
     """
     Test miscellaneous functions in llvm.binding.
@@ -250,6 +558,18 @@ class TestMisc(BaseTest):
         self.assertIn("parsing error", s)
         self.assertIn("invalid operand type", s)
 
+    def test_nonalphanum_block_name(self):
+        mod = ir.Module()
+        ft = ir.FunctionType(ir.IntType(32), [])
+        fn = ir.Function(mod, ft, "foo")
+        bd = ir.IRBuilder(fn.append_basic_block(name="<>!*''#"))
+        bd.ret(ir.Constant(ir.IntType(32), 12345))
+        asm = str(mod)
+        self.assertEqual(
+            clean_string_whitespace(asm),
+            clean_string_whitespace(asm_nonalphanum_blocklabel)
+        )
+
     def test_global_context(self):
         gcontext1 = llvm.context.get_global_context()
         gcontext2 = llvm.context.get_global_context()
@@ -280,23 +600,13 @@ class TestMisc(BaseTest):
         self.assertEqual(default_parts[0], triple_parts[0])
 
     def test_get_host_cpu_features(self):
-        try:
-            features = llvm.get_host_cpu_features()
-        except RuntimeError:
-            # Allow non-x86 arch to pass even if an RuntimeError is raised
-            triple = llvm.get_process_triple()
-            # For now, we know for sure that x86 is supported.
-            # We can restrict the test if we know this works on other arch.
-            is_x86 = triple.startswith('x86')
-            self.assertFalse(is_x86,
-                             msg="get_host_cpu_features() should not raise")
-            return
+        features = llvm.get_host_cpu_features()
         # Check the content of `features`
         self.assertIsInstance(features, dict)
         self.assertIsInstance(features, llvm.FeatureMap)
         for k, v in features.items():
             self.assertIsInstance(k, str)
-            self.assertTrue(k)  # feature string cannot be empty
+            self.assertTrue(k)  # single feature string cannot be empty
             self.assertIsInstance(v, bool)
         self.assertIsInstance(features.flatten(), str)
 
@@ -307,7 +617,10 @@ class TestMisc(BaseTest):
         self.assertIsNotNone(re.match(regex, "+aa"))
         self.assertIsNotNone(re.match(regex, "+a,-bb"))
         # check CpuFeature.flatten()
-        self.assertIsNotNone(re.match(regex, features.flatten()))
+        if len(features) == 0:
+            self.assertEqual(features.flatten(), "")
+        else:
+            self.assertIsNotNone(re.match(regex, features.flatten()))
 
     def test_get_host_cpu_name(self):
         cpu = llvm.get_host_cpu_name()
@@ -340,8 +653,8 @@ class TestMisc(BaseTest):
     def test_version(self):
         major, minor, patch = llvm.llvm_version_info
         # one of these can be valid
-        valid = [(8, 0), (7, 0), (7, 1)]
-        self.assertIn((major, minor), valid)
+        valid = [(11,), (12,), (13,)]
+        self.assertIn((major,), valid)
         self.assertIn(patch, range(10))
 
     def test_check_jit_execution(self):
@@ -393,6 +706,10 @@ class TestModuleRef(BaseTest):
         mod.name = "bar"
         self.assertEqual(mod.name, "bar")
 
+    def test_source_file(self):
+        mod = self.module()
+        self.assertEqual(mod.source_file, "asm_sum.c")
+
     def test_data_layout(self):
         mod = self.module()
         s = mod.data_layout
@@ -518,8 +835,8 @@ class TestModuleRef(BaseTest):
     def test_as_bitcode(self):
         mod = self.module()
         bc = mod.as_bitcode()
-        # Refer to http://llvm.org/docs/doxygen/html/ReaderWriter_8h_source.html#l00064
-        # and http://llvm.org/docs/doxygen/html/ReaderWriter_8h_source.html#l00092
+        # Refer to http://llvm.org/docs/doxygen/html/ReaderWriter_8h_source.html#l00064  # noqa E501
+        # and http://llvm.org/docs/doxygen/html/ReaderWriter_8h_source.html#l00092  # noqa E501
         bitcode_wrapper_magic = b'\xde\xc0\x17\x0b'
         bitcode_magic = b'BC'
         self.assertTrue(bc.startswith(bitcode_magic) or
@@ -529,7 +846,13 @@ class TestModuleRef(BaseTest):
         with self.assertRaises(RuntimeError) as cm:
             llvm.parse_bitcode(b"")
         self.assertIn("LLVM bitcode parsing error", str(cm.exception))
-        self.assertIn("Invalid bitcode signature", str(cm.exception))
+        # for llvm < 9
+        if llvm.llvm_version_info[0] < 9:
+            self.assertIn("Invalid bitcode signature", str(cm.exception))
+        else:
+            self.assertIn(
+                "file too small to contain bitcode header", str(cm.exception),
+            )
 
     def test_bitcode_roundtrip(self):
         # create a new context to avoid struct renaming
@@ -738,9 +1061,9 @@ class JITWithTMTestMixin(JITTestMixin):
 
     def test_emit_assembly(self):
         """Test TargetMachineRef.emit_assembly()"""
-        target_machine = self.target_machine()
+        target_machine = self.target_machine(jit=True)
         mod = self.module()
-        ee = self.jit(mod, target_machine)
+        ee = self.jit(mod, target_machine)  # noqa F841 # Keeps pointers alive
         raw_asm = target_machine.emit_assembly(mod)
         self.assertIn("sum", raw_asm)
         target_machine.set_asm_verbosity(True)
@@ -750,11 +1073,11 @@ class JITWithTMTestMixin(JITTestMixin):
 
     def test_emit_object(self):
         """Test TargetMachineRef.emit_object()"""
-        target_machine = self.target_machine()
+        target_machine = self.target_machine(jit=True)
         mod = self.module()
-        ee = self.jit(mod, target_machine)
+        ee = self.jit(mod, target_machine)  # noqa F841 # Keeps pointers alive
         code_object = target_machine.emit_object(mod)
-        self.assertIsInstance(code_object, six.binary_type)
+        self.assertIsInstance(code_object, bytes)
         if sys.platform.startswith('linux'):
             # Sanity check
             self.assertIn(b"ELF", code_object[:10])
@@ -767,7 +1090,7 @@ class TestMCJit(BaseTest, JITWithTMTestMixin):
 
     def jit(self, mod, target_machine=None):
         if target_machine is None:
-            target_machine = self.target_machine()
+            target_machine = self.target_machine(jit=True)
         return llvm.create_mcjit_compiler(mod, target_machine)
 
 
@@ -896,11 +1219,11 @@ class TestValueRef(BaseTest):
         self.assertTrue(func.is_function)
         self.assertEqual(func.name, 'sum')
 
-        with self.assertRaises(ValueError) as cm:
+        with self.assertRaises(ValueError):
             func.instructions
-        with self.assertRaises(ValueError) as cm:
+        with self.assertRaises(ValueError):
             func.operands
-        with self.assertRaises(ValueError) as cm:
+        with self.assertRaises(ValueError):
             func.opcode
 
     def test_function_arguments(self):
@@ -916,9 +1239,9 @@ class TestValueRef(BaseTest):
         self.assertEqual(args[1].name, '.2')
         self.assertEqual(str(args[1].type), 'i32')
 
-        with self.assertRaises(ValueError) as cm:
+        with self.assertRaises(ValueError):
             args[0].blocks
-        with self.assertRaises(ValueError) as cm:
+        with self.assertRaises(ValueError):
             args[0].arguments
 
     def test_function_blocks(self):
@@ -954,7 +1277,6 @@ class TestValueRef(BaseTest):
 
     def test_function_attributes(self):
         mod = self.module(asm_attributes)
-        funcs = list(mod.functions)
         for func in mod.functions:
             attrs = list(func.attributes)
             if func.name == 'a_readonly_func':
@@ -1040,12 +1362,12 @@ class TestTargetData(BaseTest):
 class TestTargetMachine(BaseTest):
 
     def test_add_analysis_passes(self):
-        tm = self.target_machine()
+        tm = self.target_machine(jit=False)
         pm = llvm.create_module_pass_manager()
         tm.add_analysis_passes(pm)
 
     def test_target_data_from_tm(self):
-        tm = self.target_machine()
+        tm = self.target_machine(jit=False)
         td = tm.target_data
         mod = self.module()
         gv_i32 = mod.get_global_variable("glob")
@@ -1072,14 +1394,14 @@ class TestPassManagerBuilder(BaseTest):
 
     def test_opt_level(self):
         pmb = self.pmb()
-        self.assertIsInstance(pmb.opt_level, six.integer_types)
+        self.assertIsInstance(pmb.opt_level, int)
         for i in range(4):
             pmb.opt_level = i
             self.assertEqual(pmb.opt_level, i)
 
     def test_size_level(self):
         pmb = self.pmb()
-        self.assertIsInstance(pmb.size_level, six.integer_types)
+        self.assertIsInstance(pmb.size_level, int)
         for i in range(4):
             pmb.size_level = i
             self.assertEqual(pmb.size_level, i)
@@ -1133,6 +1455,7 @@ class PassManagerTestMixin(object):
     def pmb(self):
         pmb = llvm.create_pass_manager_builder()
         pmb.opt_level = 2
+        pmb.inlining_threshold = 300
         return pmb
 
     def test_close(self):
@@ -1168,7 +1491,6 @@ class TestModulePassManager(BaseTest, PassManagerTestMixin):
                         break
                 else:
                     raise RuntimeError("expected tokens not found")
-                add_line = opt_asm_split[idx]
                 othertoken = (toks ^ {t}).pop()
 
                 self.assertIn("%.3", orig_asm)
@@ -1177,6 +1499,49 @@ class TestModulePassManager(BaseTest, PassManagerTestMixin):
         else:
             raise RuntimeError("expected IR not found")
 
+    def test_run_with_remarks_successful_inline(self):
+        pm = self.pm()
+        pm.add_function_inlining_pass(70)
+        self.pmb().populate(pm)
+        mod = self.module(asm_inlineasm2)
+        (status, remarks) = pm.run_with_remarks(mod)
+        self.assertTrue(status)
+        # Inlining has happened?  The remark will tell us.
+        self.assertIn("Passed", remarks)
+        self.assertIn("inlineme", remarks)
+
+    def test_run_with_remarks_failed_inline(self):
+        pm = self.pm()
+        pm.add_function_inlining_pass(0)
+        self.pmb().populate(pm)
+        mod = self.module(asm_inlineasm3)
+        (status, remarks) = pm.run_with_remarks(mod)
+        self.assertTrue(status)
+
+        # Inlining has not happened?  The remark will tell us.
+        self.assertIn("Missed", remarks)
+        self.assertIn("inlineme", remarks)
+        self.assertIn("noinline function attribute", remarks)
+
+    def test_run_with_remarks_inline_filter_out(self):
+        pm = self.pm()
+        pm.add_function_inlining_pass(70)
+        self.pmb().populate(pm)
+        mod = self.module(asm_inlineasm2)
+        (status, remarks) = pm.run_with_remarks(mod, remarks_filter="nothing")
+        self.assertTrue(status)
+        self.assertEqual("", remarks)
+
+    def test_run_with_remarks_inline_filter_in(self):
+        pm = self.pm()
+        pm.add_function_inlining_pass(70)
+        self.pmb().populate(pm)
+        mod = self.module(asm_inlineasm2)
+        (status, remarks) = pm.run_with_remarks(mod, remarks_filter="inlin.*")
+        self.assertTrue(status)
+        self.assertIn("Passed", remarks)
+        self.assertIn("inlineme", remarks)
+
 
 class TestFunctionPassManager(BaseTest, PassManagerTestMixin):
 
@@ -1204,6 +1569,50 @@ class TestFunctionPassManager(BaseTest, PassManagerTestMixin):
         self.assertIn("%.4", orig_asm)
         self.assertNotIn("%.4", opt_asm)
 
+    def test_run_with_remarks(self):
+        mod = self.module(licm_asm)
+        fn = mod.get_function("licm")
+        pm = self.pm(mod)
+        pm.add_licm_pass()
+        self.pmb().populate(pm)
+        mod.close()
+
+        pm.initialize()
+        (ok, remarks) = pm.run_with_remarks(fn)
+        pm.finalize()
+        self.assertTrue(ok)
+        self.assertIn("Passed", remarks)
+        self.assertIn("licm", remarks)
+
+    def test_run_with_remarks_filter_out(self):
+        mod = self.module(licm_asm)
+        fn = mod.get_function("licm")
+        pm = self.pm(mod)
+        pm.add_licm_pass()
+        self.pmb().populate(pm)
+        mod.close()
+
+        pm.initialize()
+        (ok, remarks) = pm.run_with_remarks(fn, remarks_filter="nothing")
+        pm.finalize()
+        self.assertTrue(ok)
+        self.assertEqual("", remarks)
+
+    def test_run_with_remarks_filter_in(self):
+        mod = self.module(licm_asm)
+        fn = mod.get_function("licm")
+        pm = self.pm(mod)
+        pm.add_licm_pass()
+        self.pmb().populate(pm)
+        mod.close()
+
+        pm.initialize()
+        (ok, remarks) = pm.run_with_remarks(fn, remarks_filter="licm")
+        pm.finalize()
+        self.assertTrue(ok)
+        self.assertIn("Passed", remarks)
+        self.assertIn("licm", remarks)
+
 
 class TestPasses(BaseTest, PassManagerTestMixin):
 
@@ -1228,6 +1637,7 @@ class TestPasses(BaseTest, PassManagerTestMixin):
         pm.add_sroa_pass()
         pm.add_type_based_alias_analysis_pass()
         pm.add_basic_alias_analysis_pass()
+        pm.add_loop_rotate_pass()
 
 
 class TestDylib(BaseTest):
@@ -1399,7 +1809,7 @@ class TestInlineAsm(BaseTest):
     def test_inlineasm(self):
         llvm.initialize_native_asmparser()
         m = self.module(asm=asm_inlineasm)
-        tm = self.target_machine()
+        tm = self.target_machine(jit=False)
         asm = tm.emit_assembly(m)
         self.assertIn('nop', asm)
 
@@ -1420,7 +1830,7 @@ class TestObjectFile(BaseTest):
     """
 
     def test_object_file(self):
-        target_machine = self.target_machine()
+        target_machine = self.target_machine(jit=False)
         mod = self.module()
         obj_bin = target_machine.emit_object(mod)
         obj = llvm.ObjectFileRef.from_data(obj_bin)
@@ -1440,13 +1850,13 @@ class TestObjectFile(BaseTest):
         self.assertTrue(has_text)
 
     def test_add_object_file(self):
-        target_machine = self.target_machine()
+        target_machine = self.target_machine(jit=False)
         mod = self.module()
         obj_bin = target_machine.emit_object(mod)
         obj = llvm.ObjectFileRef.from_data(obj_bin)
 
         jit = llvm.create_mcjit_compiler(self.module(self.mod_asm),
-            target_machine)
+                                         target_machine)
 
         jit.add_object_file(obj)
 
@@ -1456,7 +1866,7 @@ class TestObjectFile(BaseTest):
         self.assertEqual(sum_twice(2, 3), 10)
 
     def test_add_object_file_from_filesystem(self):
-        target_machine = self.target_machine()
+        target_machine = self.target_machine(jit=False)
         mod = self.module()
         obj_bin = target_machine.emit_object(mod)
         temp_desc, temp_path = mkstemp()
@@ -1470,7 +1880,7 @@ class TestObjectFile(BaseTest):
                 f.close()
 
             jit = llvm.create_mcjit_compiler(self.module(self.mod_asm),
-                target_machine)
+                                             target_machine)
 
             jit.add_object_file(temp_path)
         finally:
@@ -1481,6 +1891,70 @@ class TestObjectFile(BaseTest):
 
         self.assertEqual(sum_twice(2, 3), 10)
 
+    def test_get_section_content(self):
+        # See Issue #632 - section contents were getting truncated at null
+        # bytes.
+        elf = bytes.fromhex(issue_632_elf)
+        obj = llvm.ObjectFileRef.from_data(elf)
+        for s in obj.sections():
+            if s.is_text():
+                self.assertEqual(len(s.data()), 31)
+                self.assertEqual(s.data().hex(), issue_632_text)
+
+
+class TestTimePasses(BaseTest):
+    def test_reporting(self):
+        mp = llvm.create_module_pass_manager()
+
+        pmb = llvm.create_pass_manager_builder()
+        pmb.opt_level = 3
+        pmb.populate(mp)
+
+        try:
+            llvm.set_time_passes(True)
+            mp.run(self.module())
+            mp.run(self.module())
+            mp.run(self.module())
+        finally:
+            report = llvm.report_and_reset_timings()
+            llvm.set_time_passes(False)
+
+        self.assertIsInstance(report, str)
+        self.assertEqual(report.count("Pass execution timing report"), 1)
+
+    def test_empty_report(self):
+        # Returns empty str if no data is collected
+        self.assertFalse(llvm.report_and_reset_timings())
+
+
+class TestLLVMLockCallbacks(BaseTest):
+    def test_lock_callbacks(self):
+        events = []
+
+        def acq():
+            events.append('acq')
+
+        def rel():
+            events.append('rel')
+
+        # register callback
+        llvm.ffi.register_lock_callback(acq, rel)
+
+        # Check: events are initially empty
+        self.assertFalse(events)
+        # Call LLVM functions
+        llvm.create_module_pass_manager()
+        # Check: there must be at least one acq and one rel
+        self.assertIn("acq", events)
+        self.assertIn("rel", events)
+
+        # unregister callback
+        llvm.ffi.unregister_lock_callback(acq, rel)
+
+        # Check: removing non-existent callbacks will trigger a ValueError
+        with self.assertRaises(ValueError):
+            llvm.ffi.unregister_lock_callback(acq, rel)
+
 
 if __name__ == "__main__":
     unittest.main()
